# Only native build is supported.
#

%define _binutils_version 2.42

Summary: A GNU collection of gprofng utilities
Name: binutils-gprofng
Version: 0.2.42
Release: 1%{?dist}
License: GPLv3+
URL: https://sourceware.org/binutils

#----------------------------------------------------------------------------

# Binutils SPEC file.  Can be invoked with the following parameters to change
#  the default behaviour:
#
# --with debug           Build without optimizations and without splitting
#                         the debuginfo into a separate file.
# --without docs         Skip building documentation.
#                         Default is with docs, except when building a cross binutils.

# Default: Not debug
%bcond_with debug
# Default: Always build documentation.
%bcond_without docs

# No lto optimization:
%global _lto_cflags %{nil}

#----------------------------------------------------------------------------
Source: https://ftp.gnu.org/gnu/binutils/binutils-%{_binutils_version}.tar.xz

# The patches below are backports from upstream
Patch01: 0001-gprofng-Remove-unused-macros.patch
Patch02: 0002-gprofng-Add-hardware-counter-profiling-for-the-Amper.patch
Patch03: 0003-gprofng-change-use-of-bignum-to-use-of-bigint.patch

# This is a specific patch for RPM
Patch99: 0099-Use-libopcodes.a-libbfd.a-libsframe.a.patch

BuildRequires: sed, coreutils, gcc, make, perl, findutils, texinfo
BuildRequires: gettext, flex, zlib-devel, libzstd-devel
BuildRequires: bison >= 3.0.4
BuildRequires: java-17-openjdk-devel

%if %{with debug}
# Define this if you want to skip the strip step and preserve debug info.
# Useful for testing.
%define __debug_install_post : > %{_builddir}/%{?buildsubdir}/debugfiles.list
%define debug_package %{nil}
%endif

Provides: binutils-gprofng = %{version}-%{release}
Requires: coreutils, perl

%description
This package contains the gprofng applications and libraries.


#----------------------------------------------------------------------------
%prep
%ifnarch x86_64 aarch64
echo "gprofng is not supported in %{_target_platform}"
exit 1
%endif

%autosetup -p1 -n binutils-%{_binutils_version}

# On aarch64, we might use 64KiB pages
sed -i -e '/#define.*ELF_COMMONPAGESIZE/s/0x1000$/0x10000/' bfd/elf*ppc.c
sed -i -e '/#define.*ELF_COMMONPAGESIZE/s/0x1000$/0x10000/' bfd/elf*aarch64.c
# LTP sucks
perl -pi -e 's/i\[3-7\]86/i[34567]86/g' */conf*
sed -i -e 's/%''{release}/%{release}/g' bfd/Makefile{.am,.in}
sed -i -e '/^libopcodes_la_\(DEPENDENCIES\|LIBADD\)/s,$, ../bfd/libbfd.la,' opcodes/Makefile.{am,in}
# Build libbfd.so and libopcodes.so with -Bsymbolic-functions if possible.
if gcc %{optflags} -v --help 2>&1 | grep -q -- -Bsymbolic-functions; then
sed -i -e 's/^libbfd_la_LDFLAGS = /&-Wl,-Bsymbolic-functions /' bfd/Makefile.{am,in}
sed -i -e 's/^libopcodes_la_LDFLAGS = /&-Wl,-Bsymbolic-functions /' opcodes/Makefile.{am,in}
fi
# $PACKAGE is used for the gettext catalog name.
sed -i -e 's/^ PACKAGE=/ PACKAGE=%{?cross}/' */configure

# Update the gprofng version
sed -i -e 's/2\.40\.00/%{version}-%{release}/' gprofng/Makefile{.am,.in} \
  gprofng/*/Makefile{.am,.in} gprofng/configure gprofng/*/configure \
  gprofng/doc/version.texi

touch */configure
# Touch the .info files so that they are newer then the .texi files and
# hence do not need to be rebuilt.  This eliminates the need for makeinfo.
# The -print is there just to confirm that the command is working.
%if %{without docs}
  find . -name *.info -print -exec touch {} \;
%endif

#----------------------------------------------------------------------------
%build

export CFLAGS="$RPM_OPT_FLAGS"

# BZ 1541027 - include the linker flags from redhat-rpm-config as well.
export LDFLAGS=$RPM_LD_FLAGS

CARGS=--quiet 
%configure \
  --disable-binutils \
  --disable-elfcpp \
  --disable-gas \
  --disable-gdb \
  --disable-gdbserver \
  --disable-gdbsupport \
  --disable-gnulib \
  --disable-gprof \
  --disable-itcl \
  --disable-ld \
  --disable-libbacktrace \
  --disable-libctf \
  --disable-libgui \
  --disable-libiconv \
  --disable-libquadmath \
  --disable-libquadmath-support \
  --disable-libstdcxx \
  --disable-tcl \
  --disable-tk \
  $CARGS \
  --build=%{_target_platform} --host=%{_target_platform} \
  --target=%{_target_platform} \
  --with-sysroot=/ \
  --enable-gprofng \
  --enable-shared \
  --with-pic \
  --enable-64-bit-bfd \
  --with-bugurl=https://sourceware.org/bugzilla \
  || cat config.log

%make_build

#----------------------------------------------------------------------------
%install
%make_install

ls -al %{buildroot}%{_infodir}/

rm -f %{buildroot}%{_libdir}/lib{bfd,sframe,opcodes}*
rm -f %{buildroot}%{_datadir}/locale/*/LC_MESSAGES/{bfd,opcodes}.mo
rm -f %{buildroot}%{_includedir}/[a-b,d,p-z]*.h
rm -f %{buildroot}%{_infodir}/{bfd,sframe-spec}.info %{buildroot}%{_infodir}/dir

ls -al %{buildroot}%{_infodir}/

# Remove the API for the gprofng collector library
rm -f %{buildroot}%{_libdir}/gprofng/libgp-collectorAPI*
rm -f %{buildroot}%{_includedir}/{collectorAPI,libcollector,libfcollector}.h

%if %{without docs}
rm -f %{buildroot}/%{_infodir}/gprofng.info %{buildroot}%{_mandir}/man1/gp*
%endif

#----------------------------------------------------------------------------
# Removing of the post section cleans up BUILDROOT
%post
exit 0

#----------------------------------------------------------------------------
%files
%{_bindir}/gprofng
%{_bindir}/gp-*
%{_libdir}/libgprofng*
%{_libdir}/gprofng/*
%{_sysconfdir}/gprofng.rc
%if %{with docs}
%{_infodir}/gprofng.info.gz
%{_mandir}/man1/gp*
%endif

## Uncomment next two lines to save sources
# %%clean
# echo NOOP

#----------------------------------------------------------------------------
%changelog

* Sun Mar 03 2024 Vladimir Mezentsev <vladimir.mezentsev@oracle.com> - 0.2.42-1
- gprofng with binutils-2.42

* Tue Apr 11 2023 Vladimir Mezentsev <vladimir.mezentsev@oracle.com> - 0.2.40-1
- First gprofng version being package

